@@ -17,12 +17,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-gst.c,v 1.135 2005/01/18 14:30:17 rbultje Exp $
+ * $Id: bacon-video-widget-gst.c,v 1.155 2005/03/01 16:53:28 rbultje Exp $
+ *
+ *
+ * The Totem project hereby grant permission for non-gpl compatible GStreamer
+ * plugins to be used and distributed together with GStreamer and Totem. This
+ * permission are above and beyond the permissions granted by the GPL license
+ * Totem is covered by.
+ *
+ * Monday 7th February 2005: Christian Schaller: Add excemption clause.
+ * See license_change file for details.
  *
  */
 
 #include <config.h>
 
+#ifdef HAVE_NVTV
+#include <nvtv_simple.h>
+#endif 
+
 /* gstgconf */
 #include <gst/gconf/gconf.h>
 
@@ -133,6 +146,7 @@ struct BaconVideoWidgetPrivate
   int xpos, ypos;
   gboolean logo_mode;
   gboolean cursor_shown;
+  gboolean fullscreen_mode;
   gboolean auto_resize;
   
   GAsyncQueue *queue;
@@ -158,7 +172,7 @@ enum {
   ASYNC_VIDEO_SIZE,
   ASYNC_ERROR,
   ASYNC_FOUND_TAG,
-  ASYNC_CLEAR_TAGS,
+  ASYNC_NOTIFY_STREAMINFO,
   ASYNC_EOS,
   ASYNC_BUFFERING
 };
@@ -283,6 +297,25 @@ bacon_video_widget_realize (GtkWidget * widget)
   /* nice hack to show the logo fullsize, while still being resizable */
   get_media_size (BACON_VIDEO_WIDGET (widget), &w, &h);
   totem_widget_set_preferred_size (widget, w, h);
+
+#ifdef HAVE_NVTV
+  if (!(nvtv_simple_init() && nvtv_enable_autoresize(TRUE))) {
+    nvtv_simple_enable(FALSE);
+  } 
+#endif
+
+}
+
+static void
+bacon_video_widget_unrealize (GtkWidget *widget)
+{
+#ifdef HAVE_NVTV
+  /* Kill the TV out */
+  nvtv_simple_exit();
+#endif
+
+  if (GTK_WIDGET_CLASS (parent_class)->unrealize)
+    GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
 }
 
 static gboolean
@@ -532,6 +565,7 @@ bacon_video_widget_class_init (BaconVideoWidgetClass * klass)
   widget_class->size_request = bacon_video_widget_size_request;
   widget_class->size_allocate = bacon_video_widget_size_allocate;
   widget_class->realize = bacon_video_widget_realize;
+  widget_class->unrealize = bacon_video_widget_unrealize;
   widget_class->expose_event = bacon_video_widget_expose_event;
   widget_class->motion_notify_event = bacon_video_widget_motion_notify;
   widget_class->button_press_event = bacon_video_widget_button_press;
@@ -670,9 +704,11 @@ bacon_video_widget_init (BaconVideoWidget * bvw)
 static void
 shrink_toplevel (BaconVideoWidget * bvw)
 {
-  GtkWidget *toplevel;
-  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (bvw));
-  gtk_window_resize (GTK_WINDOW (toplevel), 1, 1);
+  GtkWidget *toplevel, *widget;
+  widget = GTK_WIDGET (bvw);
+  toplevel = gtk_widget_get_toplevel (widget);
+  if (toplevel != widget && GTK_IS_WINDOW (toplevel) != FALSE)
+    gtk_window_resize (GTK_WINDOW (toplevel), 1, 1);
 }
 
 static gboolean
@@ -694,7 +730,7 @@ bacon_video_widget_signal_idler (BaconVideoWidget *bvw)
         {
           g_signal_emit (G_OBJECT (bvw), bvw_table_signals[SIGNAL_GOT_METADATA], 0, NULL);
 
-          if (bvw->priv->auto_resize) {
+          if (bvw->priv->auto_resize && !bvw->priv->fullscreen_mode) {
             gint w, h;
 
             shrink_toplevel (bvw);
@@ -767,12 +803,11 @@ bacon_video_widget_signal_idler (BaconVideoWidget *bvw)
 			 0, NULL);
           break;
         }
-      case ASYNC_CLEAR_TAGS:
+      case ASYNC_NOTIFY_STREAMINFO:
         {
-          gst_tag_list_free (bvw->priv->tagcache);
-          bvw->priv->tagcache = NULL;
           g_signal_emit (G_OBJECT (bvw),
 			 bvw_table_signals[SIGNAL_GOT_METADATA], 0, NULL);
+          g_signal_emit (bvw, bvw_table_signals[SIGNAL_CHANNELS_CHANGE], 0);
           break;
         }
       case ASYNC_EOS:
@@ -806,7 +841,7 @@ got_found_tag (GstElement *play, GstElement *source,
   
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
-  
+
   signal = g_new0 (BVWSignal, 1);
   signal->signal_id = ASYNC_FOUND_TAG;
   signal->signal_data.found_tag.source = source;
@@ -825,8 +860,11 @@ group_switch (GstElement *play, BaconVideoWidget *bvw)
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 
+  gst_tag_list_free (bvw->priv->tagcache);
+  bvw->priv->tagcache = NULL;
+
   signal = g_new0 (BVWSignal, 1);
-  signal->signal_id = ASYNC_CLEAR_TAGS;
+  signal->signal_id = ASYNC_NOTIFY_STREAMINFO;
 
   g_async_queue_push (bvw->priv->queue, signal);
 
@@ -938,6 +976,11 @@ got_source (GObject    *play,
   GObject *source = NULL;
   GObjectClass *klass;
 
+  if (bvw->priv->tagcache) {
+    gst_tag_list_free (bvw->priv->tagcache);
+    bvw->priv->tagcache = NULL;
+  }
+
   if (!bvw->priv->media_device)
     return;
 
@@ -960,7 +1003,7 @@ cb_iterate (BaconVideoWidget *bvw)
 
   /* check length/pos of stream */
   if (gst_element_query (GST_ELEMENT (bvw->priv->play),
-			 GST_QUERY_TOTAL, &fmt, &value) &
+			 GST_QUERY_TOTAL, &fmt, &value) &&
       GST_CLOCK_TIME_IS_VALID (value) &&
       value / GST_MSECOND != bvw->priv->stream_length) {
     got_stream_length (GST_ELEMENT (bvw->priv->play),
@@ -989,9 +1032,10 @@ caps_set (GObject * obj,
   if (s) {
     const GValue *par;
 
-    gst_structure_get_double (s, "framerate", &bvw->priv->video_fps);
-    gst_structure_get_int (s, "width", &bvw->priv->video_width);
-    gst_structure_get_int (s, "height", &bvw->priv->video_height);
+    if (!(gst_structure_get_double (s, "framerate", &bvw->priv->video_fps) &&
+          gst_structure_get_int (s, "width", &bvw->priv->video_width) &&
+          gst_structure_get_int (s, "height", &bvw->priv->video_height)))
+      return;
     if ((par = gst_structure_get_value (s,
                    "pixel-aspect-ratio"))) {
       gint num = gst_value_get_fraction_numerator (par),
@@ -1002,10 +1046,88 @@ caps_set (GObject * obj,
       else
         bvw->priv->video_height *= (gfloat) den / num;
     }
+
+    got_video_size (bvw->priv->play, bvw->priv->video_width,
+		    bvw->priv->video_height, bvw);
   }
 
   /* and disable ourselves */
-  g_signal_handlers_disconnect_by_func (pad, caps_set, bvw);
+  //g_signal_handlers_disconnect_by_func (pad, caps_set, bvw);
+}
+
+static GstCaps * fixate_visualization (GstPad *pad, const GstCaps *in_caps,
+				       BaconVideoWidget *bvw);
+
+static void
+parse_stream_info (BaconVideoWidget *bvw)
+{
+  GList *streaminfo = NULL;
+  GstPad *videopad = NULL;
+
+  g_object_get (G_OBJECT (bvw->priv->play), "stream-info",
+		&streaminfo, NULL);
+  streaminfo = g_list_copy (streaminfo);
+  g_list_foreach (streaminfo, (GFunc) g_object_ref, NULL);
+  for ( ; streaminfo != NULL; streaminfo = streaminfo->next) {
+    GObject *info = streaminfo->data;
+    gint type;
+    GParamSpec *pspec;
+    GEnumValue *val;
+
+    if (!info)
+      continue;
+    g_object_get (info, "type", &type, NULL);
+    pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (info), "type");
+    val = g_enum_get_value (G_PARAM_SPEC_ENUM (pspec)->enum_class, type);
+
+    if (strstr (val->value_name, "AUDIO")) {
+      if (!bvw->priv->media_has_audio) {
+        bvw->priv->media_has_audio = TRUE;
+        /*if (!bvw->priv->media_has_video &&
+            bvw->priv->show_vfx && bvw->priv->vis_element) {
+          videopad = gst_element_get_pad (bvw->priv->vis_element, "src");
+        }*/
+      }
+    } else if (strstr (val->value_name, "VIDEO")) {
+      bvw->priv->media_has_video = TRUE;
+      if (!videopad) {
+        g_object_get (info, "object", &videopad, NULL);
+        /* hack for 0.8.5 */
+        if (!videopad)
+          g_object_get (info, "pad", &videopad, NULL);
+      }
+    }
+  }
+
+  if (videopad) {
+    GstPad *real = (GstPad *) GST_PAD_REALIZE (videopad);
+
+    /* handle explicit caps as well - they're set later */
+    if (((GstRealPad *) real)->link != NULL && GST_PAD_CAPS (real))
+      caps_set (G_OBJECT (real), NULL, bvw);
+    //else
+      g_signal_connect (real, "notify::caps",
+          G_CALLBACK (caps_set), bvw);
+  } else if (bvw->priv->show_vfx && bvw->priv->vis_element) {
+    fixate_visualization (NULL, NULL, bvw);
+  }
+  g_list_foreach (streaminfo, (GFunc) g_object_unref, NULL);
+  g_list_free (streaminfo);
+}
+
+static void
+stream_info_set (GObject * obj, GParamSpec * pspec, BaconVideoWidget * bvw)
+{
+  BVWSignal *signal;
+
+  parse_stream_info (bvw);
+
+  signal = g_new0 (BVWSignal, 1);
+  signal->signal_id = ASYNC_NOTIFY_STREAMINFO;
+
+  g_async_queue_push (bvw->priv->queue, signal);
+
+  g_idle_add ((GSourceFunc) bacon_video_widget_signal_idler, bvw);
 }
 
 static void
@@ -1025,52 +1147,7 @@ state_change (GstElement *play, GstElementState old_state,
 
   if (old_state <= GST_STATE_READY &&
       new_state >= GST_STATE_PAUSED) {
-    GList *streaminfo = NULL;
-    GstPad *videopad = NULL;
-
-    g_object_get (G_OBJECT (bvw->priv->play), "stream-info",
-		  &streaminfo, NULL);
-    for ( ; streaminfo != NULL; streaminfo = streaminfo->next) {
-      GObject *info = streaminfo->data;
-      gint type;
-      GParamSpec *pspec;
-      GEnumValue *val;
-
-      g_object_get (info, "type", &type, NULL);
-      pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (info), "type");
-      val = g_enum_get_value (G_PARAM_SPEC_ENUM (pspec)->enum_class, type);
-
-      if (strstr (val->value_name, "AUDIO")) {
-        if (!bvw->priv->media_has_audio) {
-          bvw->priv->media_has_audio = TRUE;
-          if (!bvw->priv->media_has_video &&
-              bvw->priv->show_vfx && bvw->priv->vis_element) {
-            videopad = gst_element_get_pad (bvw->priv->vis_element, "src");
-          }
-        }
-      } else if (strstr (val->value_name, "VIDEO")) {
-        if (!bvw->priv->media_has_video) {
-          bvw->priv->media_has_video = TRUE;
-
-          videopad = NULL;
-          g_object_get (info, "object", &videopad, NULL);
-          /* hack for 0.8.5 */
-          if (!videopad)
-            g_object_get (info, "pad", &videopad, NULL);
-        }
-      }
-    }
-
-    if (videopad) {
-      GstPad *real = (GstPad *) GST_PAD_REALIZE (videopad);
-
-      /* handle explicit caps as well - they're set later */
-      if (((GstRealPad *) real)->link != NULL && GST_PAD_CAPS (real))
-        caps_set (G_OBJECT (real), NULL, bvw);
-      else
-        g_signal_connect (real, "notify::caps",
-            G_CALLBACK (caps_set), bvw);
-    }
+    parse_stream_info (bvw);
   } else if (new_state <= GST_STATE_READY &&
              old_state >= GST_STATE_PAUSED) {
     bvw->priv->media_has_video = FALSE;
@@ -1232,16 +1309,23 @@ bacon_video_widget_set_subtitle (BaconVideoWidget * bvw, int subtitle)
 static GList *
 get_list_of_type (BaconVideoWidget * bvw, const gchar * type_name)
 {
-  GList *streaminfo, *ret = NULL;
+  GList *streaminfo = NULL, *ret = NULL;
   gint num = 0;
 
+  if (bvw->priv->play == NULL || bvw->priv->mrl == NULL)
+    return NULL;
+
   g_object_get (G_OBJECT (bvw->priv->play), "stream-info", &streaminfo, NULL);
+  streaminfo = g_list_copy (streaminfo);
+  g_list_foreach (streaminfo, (GFunc) g_object_ref, NULL);
   for ( ; streaminfo != NULL; streaminfo = streaminfo->next) {
     GObject *info = streaminfo->data;
     gint type;
     GParamSpec *pspec;
     GEnumValue *val;
 
+    if (!info)
+      continue;
     g_object_get (info, "type", &type, NULL);
     pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (info), "type");
     val = g_enum_get_value (G_PARAM_SPEC_ENUM (pspec)->enum_class, type);
@@ -1250,17 +1334,27 @@ get_list_of_type (BaconVideoWidget * bvw, const gchar * type_name)
       ret = g_list_prepend (ret, g_strdup_printf ("%s %d", type_name, num++));
     }
   }
+  g_list_foreach (streaminfo, (GFunc) g_object_unref, NULL);
+  g_list_free (streaminfo);
 
   return g_list_reverse (ret);
 }
 
 GList * bacon_video_widget_get_subtitles (BaconVideoWidget * bvw)
 {
+  g_return_val_if_fail (bvw != NULL, NULL);
+  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), NULL);
+  g_return_val_if_fail (bvw->priv->play != NULL, NULL);
+
   return get_list_of_type (bvw, "TEXT");
 }
 
 GList * bacon_video_widget_get_languages (BaconVideoWidget * bvw)
 {
+  g_return_val_if_fail (bvw != NULL, NULL);
+  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), NULL);
+  g_return_val_if_fail (bvw->priv->play != NULL, NULL);
+
   return get_list_of_type (bvw, "AUDIO");
 }
 
@@ -1338,7 +1432,15 @@ bacon_video_widget_set_tv_out (BaconVideoWidget * bvw, TvOutType tvout)
   gconf_client_set_int (bvw->priv->gc,
       GCONF_PREFIX"/tv_out_type", tvout, NULL);
 
-  return TRUE;
+#ifdef HAVE_NVTV
+  if (tvout == TV_OUT_NVTV_PAL) {
+    nvtv_simple_set_tvsystem(NVTV_SIMPLE_TVSYSTEM_PAL);
+  } else if (tvout == TV_OUT_NVTV_NTSC) {
+    nvtv_simple_set_tvsystem(NVTV_SIMPLE_TVSYSTEM_NTSC);
+  }
+#endif
+
+  return FALSE;
 }
 
 TvOutType
@@ -1460,17 +1562,17 @@ bacon_video_widget_get_audio_out_type (BaconVideoWidget *bvw)
   return bvw->priv->speakersetup;
 }
 
-void
+gboolean
 bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
                                        BaconVideoWidgetAudioOutType type)
 {
   GstElement *audiosink = NULL;
 
-  g_return_if_fail (bvw != NULL);
-  g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+  g_return_val_if_fail (bvw != NULL, FALSE);
+  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
 
   if (type == bvw->priv->speakersetup)
-    return;
+    return FALSE;
 
   bvw->priv->speakersetup = type;
   gconf_client_set_int (bvw->priv->gc,
@@ -1480,6 +1582,8 @@ bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
   if (GST_STATE (bvw->priv->play) >= GST_STATE_PAUSED) {
     //gst_pad_renegotiate (gst_element_get_pad (audiosink, "sink"));
   }
+
+  return FALSE;
 }
 
 /* =========================================== */
@@ -1778,8 +1882,12 @@ bacon_video_widget_set_volume (BaconVideoWidget * bvw, int volume)
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
   g_return_if_fail (GST_IS_ELEMENT (bvw->priv->play));
 
-  g_object_set (G_OBJECT (bvw->priv->play), "volume",
-		(gdouble) (1. * volume / 100), NULL);
+  if (bacon_video_widget_can_set_volume (bvw) != FALSE)
+  {
+    volume = CLAMP (volume, 0, 100);
+    g_object_set (G_OBJECT (bvw->priv->play), "volume",
+	(gdouble) (1. * volume / 100), NULL);
+  }
 }
 
 int
@@ -1806,7 +1914,16 @@ bacon_video_widget_fullscreen_mode_available (BaconVideoWidget *bvw,
 		return TRUE;
 	case TV_OUT_NVTV_NTSC:
 	case TV_OUT_NVTV_PAL:
+#ifdef HAVE_NVTV
+		/* Make sure nvtv is initialized, it will not do any harm 
+		 * if it is done twice any way */
+		if (!(nvtv_simple_init() && nvtv_enable_autoresize(TRUE))) {
+			nvtv_simple_enable(FALSE);
+		}
+		return (nvtv_simple_is_available());
+#else
 		return FALSE;
+#endif
 	case TV_OUT_DXR3:
 		/* FIXME: Add DXR3 detection code */
 		return FALSE;
@@ -1820,6 +1937,29 @@ bacon_video_widget_set_fullscreen (BaconVideoWidget * bvw,
 {
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+
+  bvw->priv->fullscreen_mode = fullscreen;
+
+#ifdef HAVE_NVTV
+  if (bvw->priv->tv_out_type != TV_OUT_NVTV_NTSC &&
+      bvw->priv->tv_out_type != TV_OUT_NVTV_PAL)
+    return;
+
+  if (fullscreen == FALSE)
+  {
+    /* If NVTV is used */
+    if (nvtv_simple_get_state() == NVTV_SIMPLE_TV_ON) {
+      nvtv_simple_switch(NVTV_SIMPLE_TV_OFF,0,0);
+
+    }
+    /* Turn fullscreen on with NVTV if that option is on */
+  } else if ((bvw->priv->tv_out_type == TV_OUT_NVTV_NTSC) ||
+      (bvw->priv->tv_out_type == TV_OUT_NVTV_PAL)) {
+    nvtv_simple_switch(NVTV_SIMPLE_TV_ON,
+	bvw->priv->video_width,
+	bvw->priv->video_height);
+  }
+#endif
 }
 
 void
@@ -1895,6 +2035,11 @@ fixate_visualization (GstPad *pad, const GstCaps *in_caps,
       g_assert_not_reached ();
   }
 
+  bvw->priv->video_width = w;
+  bvw->priv->video_height = h;
+  if (!in_caps)
+    return NULL;
+
   /* simple please */
   if (gst_caps_get_size (in_caps) > 1)
     return NULL;
@@ -2174,16 +2319,17 @@ bacon_video_widget_get_video_property (BaconVideoWidget *bvw,
   g_return_val_if_fail (bvw != NULL, value);
   g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), value);
   
-  if (GST_IS_COLOR_BALANCE (bvw->priv->balance))
+  if (bvw->priv->balance && GST_IS_COLOR_BALANCE (bvw->priv->balance))
     {
       const GList *channels_list = NULL;
       GstColorBalanceChannel *found_channel = NULL;
       
       channels_list = gst_color_balance_list_channels (bvw->priv->balance);
       
-      while (channels_list)
+      while (channels_list != NULL && found_channel == NULL)
         { /* We search for the right channel corresponding to type */
           GstColorBalanceChannel *channel = channels_list->data;
+
           if (type == BVW_VIDEO_BRIGHTNESS && channel &&
               g_strrstr (channel->label, "BRIGHTNESS"))
             {
@@ -2211,17 +2357,20 @@ bacon_video_widget_get_video_property (BaconVideoWidget *bvw,
           channels_list = g_list_next (channels_list);
         }
         
-      if (GST_IS_COLOR_BALANCE_CHANNEL (found_channel))
+      if (found_channel && GST_IS_COLOR_BALANCE_CHANNEL (found_channel))
         {
           value = gst_color_balance_get_value (bvw->priv->balance,
                                                found_channel);
           value = ((double) value - found_channel->min_value) * 65535 /
                   ((double) found_channel->max_value - found_channel->min_value);
           g_object_unref (found_channel);
+
+	  return value;
         }
     }
-    
-  return value;
+
+  /* value wasn't found, get from gconf */
+  return gconf_client_get_int (bvw->priv->gc, video_props_str[type], NULL);
 }
 
 void
@@ -2234,17 +2383,18 @@ bacon_video_widget_set_video_property (BaconVideoWidget *bvw,
   
   if ( !(value < 65535 && value > 0) )
     return;
-  
-  if (GST_IS_COLOR_BALANCE (bvw->priv->balance))
+
+  if (bvw->priv->balance && GST_IS_COLOR_BALANCE (bvw->priv->balance))
     {
       const GList *channels_list = NULL;
       GstColorBalanceChannel *found_channel = NULL;
       
       channels_list = gst_color_balance_list_channels (bvw->priv->balance);
-      
-      while (channels_list)
-        { /* We search for the right channel corresponding to type */
+
+      while (found_channel == NULL && channels_list != NULL) {
+          /* We search for the right channel corresponding to type */
           GstColorBalanceChannel *channel = channels_list->data;
+
           if (type == BVW_VIDEO_BRIGHTNESS && channel &&
               g_strrstr (channel->label, "BRIGHTNESS"))
             {
@@ -2271,12 +2421,13 @@ bacon_video_widget_set_video_property (BaconVideoWidget *bvw,
             }
           channels_list = g_list_next (channels_list);
         }
-        
-      if (GST_IS_COLOR_BALANCE_CHANNEL (found_channel))
+
+      if (found_channel && GST_IS_COLOR_BALANCE_CHANNEL (found_channel))
         {
-          value = value * ((double) found_channel->max_value - found_channel->min_value) / 65535 + found_channel->min_value;
+          int i_value = value * ((double) found_channel->max_value -
+	      found_channel->min_value) / 65535 + found_channel->min_value;
           gst_color_balance_set_value (bvw->priv->balance, found_channel,
-                                       value);
+                                       i_value);
           g_object_unref (found_channel);
         }
     }
@@ -2497,12 +2648,6 @@ bacon_video_widget_get_metadata_int (BaconVideoWidget * bvw,
           break;
         integer += t;
       }
-      for (num = 0; ; num++) {
-        if (!gst_tag_list_get_uint_index (bvw->priv->tagcache,
-					  GST_TAG_NOMINAL_BITRATE, num, &t))
-          break;
-        integer += t;
-      }
       /* kbps */
       integer /= 1000;
       break;
@@ -2621,7 +2766,7 @@ GdkPixbuf *
 bacon_video_widget_get_current_frame (BaconVideoWidget * bvw)
 {
   GstBuffer *buf = NULL;
-  const GList *streaminfo = NULL;
+  GList *streaminfo = NULL;
   GstCaps *from = NULL;
   GdkPixbuf *pixbuf;
 
@@ -2644,6 +2789,8 @@ bacon_video_widget_get_current_frame (BaconVideoWidget * bvw)
   /* get video size etc. */
   g_object_get (G_OBJECT (bvw->priv->play),
       "stream-info", &streaminfo, NULL);
+  streaminfo = g_list_copy (streaminfo);
+  g_list_foreach (streaminfo, (GFunc) g_object_ref, NULL);
   for (; streaminfo != NULL; streaminfo = streaminfo->next) {
     GObject *info = streaminfo->data;
     GstPad *real;
@@ -2651,6 +2798,8 @@ bacon_video_widget_get_current_frame (BaconVideoWidget * bvw)
     GParamSpec *pspec;
     GEnumValue *val;
 
+    if (!info)
+      continue;
     g_object_get (info, "type", &type, NULL);
     pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (info), "type");
     val = g_enum_get_value (G_PARAM_SPEC_ENUM (pspec)->enum_class, type);
@@ -2668,6 +2817,8 @@ bacon_video_widget_get_current_frame (BaconVideoWidget * bvw)
       break;
     }
   }
+  g_list_foreach (streaminfo, (GFunc) g_object_unref, NULL);
+  g_list_free (streaminfo);
   if (!from)
     return NULL;
 
@@ -2875,6 +3026,8 @@ bacon_video_widget_new (int width, int height,
 		    G_CALLBACK (got_buffering), (gpointer) bvw);
   g_signal_connect (G_OBJECT (bvw->priv->play), "notify::source",
 		    G_CALLBACK (got_source), (gpointer) bvw);
+  g_signal_connect (G_OBJECT (bvw->priv->play), "notify::stream-info",
+		    G_CALLBACK (stream_info_set), (gpointer) bvw);
   g_signal_connect (G_OBJECT (bvw->priv->play), "group-switch",
 		    G_CALLBACK (group_switch), (gpointer) bvw);
 
@@ -2945,7 +3098,8 @@ bacon_video_widget_new (int width, int height,
   /* audio out, if any */
   confvalue = gconf_client_get_without_default (bvw->priv->gc,
       GCONF_PREFIX"/audio_output_type", NULL);
-  if (confvalue != NULL) {
+  if (confvalue != NULL &&
+      (type != BVW_USE_TYPE_METADATA && type != BVW_USE_TYPE_CAPTURE)) {
     bvw->priv->speakersetup = gconf_value_get_int (confvalue);
   }
 
