@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.163 2005/01/18 14:30:17 rbultje Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.170 2005/02/21 21:57:58 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -122,7 +122,9 @@ struct BaconVideoWidgetPrivate {
 	xine_t *xine;
 	xine_stream_t *stream;
 	xine_video_port_t *vo_driver;
+	gboolean vo_driver_none;
 	xine_audio_port_t *ao_driver;
+	gboolean ao_driver_none;
 	xine_event_queue_t *ev_queue;
 	double display_ratio;
 
@@ -157,6 +159,7 @@ struct BaconVideoWidgetPrivate {
 	gboolean have_xvidmode;
 	gboolean auto_resize;
 	int volume;
+	BaconVideoWidgetAudioOutType audio_out_type;
 	TvOutType tvout;
 	gboolean is_live;
 	char *codecs_path;
@@ -240,7 +243,7 @@ static void bacon_video_widget_size_allocate (GtkWidget *widget,
 static xine_video_port_t * load_video_out_driver (BaconVideoWidget *bvw,
 		gboolean null_out);
 static xine_audio_port_t * load_audio_out_driver (BaconVideoWidget *bvw,
-		GError **error);
+		gboolean null_out, GError **error);
 static gboolean bacon_video_widget_tick_send (BaconVideoWidget *bvw);
 
 static GtkWidgetClass *parent_class = NULL;
@@ -540,6 +543,7 @@ load_video_out_driver (BaconVideoWidget *bvw, gboolean null_out)
 
 	if (null_out != FALSE)
 	{
+		bvw->priv->vo_driver_none = TRUE;
 		return xine_open_video_driver (bvw->priv->xine,
 				"none", XINE_VISUAL_TYPE_NONE, NULL);
 	}
@@ -607,11 +611,21 @@ load_video_out_driver (BaconVideoWidget *bvw, gboolean null_out)
 }
 
 static xine_audio_port_t *
-load_audio_out_driver (BaconVideoWidget *bvw, GError **error)
+load_audio_out_driver (BaconVideoWidget *bvw, gboolean null_out,
+		GError **error)
 {
 	xine_audio_port_t *ao_driver;
 	const char *audio_driver_id;
 
+	if (null_out != FALSE) {
+		ao_driver = xine_open_audio_driver (bvw->priv->xine,
+				"none", NULL);
+		if (ao_driver != NULL) {
+			bvw->priv->ao_driver_none = TRUE;
+		}
+		return ao_driver;
+	}
+
 	audio_driver_id = xine_config_register_string (bvw->priv->xine,
 			"audio.driver", "auto", "audio driver to use",
 			NULL, 10, NULL, NULL);
@@ -1098,10 +1112,11 @@ bacon_video_widget_realize (GtkWidget *widget)
 		g_idle_add ((GSourceFunc) bacon_video_widget_idle_signal, bvw);
 	}
 
-	bvw->priv->ao_driver = load_audio_out_driver (bvw, NULL);
+	bvw->priv->ao_driver = load_audio_out_driver (bvw, FALSE, NULL);
 
 	if (bvw->priv->type == BVW_USE_TYPE_VIDEO
-			&& bvw->priv->ao_driver != NULL)
+			&& bvw->priv->ao_driver != NULL
+			&& bvw->priv->ao_driver_none == FALSE)
 	{
 		bvw->priv->vis = xine_post_init (bvw->priv->xine,
 				bvw->priv->vis_name, 0,
@@ -1558,15 +1573,19 @@ bacon_video_widget_new (int width, int height,
 	/* load the output drivers */
 	if (type == BVW_USE_TYPE_AUDIO)
 	{
-		bvw->priv->ao_driver = load_audio_out_driver (bvw, error);
+		bvw->priv->ao_driver = load_audio_out_driver (bvw,
+				FALSE, error);
 		if (error != NULL && *error != NULL)
 			return NULL;
+	} else if (type == BVW_USE_TYPE_METADATA) {
+		bvw->priv->ao_driver = load_audio_out_driver (bvw,
+				TRUE, error);
 	}
 
 	/* We need to wait for the widget to realise if we want to
 	 * load a video output with screen output, and capture is the
 	 * only one actually needing a video output */
-	if (type == BVW_USE_TYPE_CAPTURE) {
+	if (type == BVW_USE_TYPE_CAPTURE || type == BVW_USE_TYPE_METADATA) {
 		bvw->priv->vo_driver = load_video_out_driver (bvw, TRUE);
 	}
 
@@ -2275,7 +2294,10 @@ bacon_video_widget_can_set_volume (BaconVideoWidget *bvw)
 	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), 0);
 	g_return_val_if_fail (bvw->priv->xine != NULL, 0);
 
-	if (bvw->priv->ao_driver == NULL)
+	if (bvw->priv->ao_driver == NULL || bvw->priv->ao_driver_none != FALSE)
+		return FALSE;
+
+	if (bvw->priv->audio_out_type == BVW_AUDIO_SOUND_AC3PASSTHRU)
 		return FALSE;
 
 	if (xine_get_param (bvw->priv->stream,
@@ -2356,6 +2378,9 @@ bacon_video_widget_fullscreen_mode_available (BaconVideoWidget *bvw,
 void
 bacon_video_widget_set_fullscreen (BaconVideoWidget *bvw, gboolean fullscreen)
 {
+	g_return_if_fail (bvw != NULL);
+	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+
 	if (bvw->priv->have_xvidmode == FALSE &&
 			bvw->priv->tvout != TV_OUT_NVTV_NTSC &&
 			bvw->priv->tvout != TV_OUT_NVTV_PAL)
@@ -2422,6 +2447,10 @@ bacon_video_widget_set_media_device (BaconVideoWidget *bvw, const char *path)
 {
 	xine_cfg_entry_t entry;
 
+	g_return_if_fail (bvw != NULL);
+	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+	g_return_if_fail (path != NULL);
+
 	/* DVD device */
 	bvw_config_helper_string (bvw->priv->xine, "media.dvd.device",
 			path, &entry);
@@ -3055,16 +3084,20 @@ bacon_video_widget_get_audio_out_type (BaconVideoWidget *bvw)
 			GCONF_PREFIX"/audio_output_type", NULL);
 }
 
-void
+gboolean
 bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
 		BaconVideoWidgetAudioOutType type)
 {
 	xine_cfg_entry_t entry;
 	int value;
+	gboolean need_restart = FALSE;
 
-	g_return_if_fail (bvw != NULL);
-	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
-	g_return_if_fail (bvw->priv->xine != NULL);
+	g_return_val_if_fail (bvw != NULL, FALSE);
+	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
+	g_return_val_if_fail (bvw->priv->xine != NULL, FALSE);
+
+	if (type == bvw->priv->audio_out_type)
+		return FALSE;
 
 	xine_config_register_enum (bvw->priv->xine,
 			"audio.output.speaker_arrangement",
@@ -3095,6 +3128,7 @@ bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
 		break;
 	case BVW_AUDIO_SOUND_AC3PASSTHRU:
 		value = 12;
+		need_restart = TRUE;
 		break;
 	default:
 		value = 1;
@@ -3105,6 +3139,8 @@ bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
 			"audio.output.speaker_arrangement", &entry);
 	entry.num_value = value;
 	xine_config_update_entry (bvw->priv->xine, &entry);
+
+	return need_restart;
 }
 
 static void
@@ -3303,10 +3339,22 @@ GList
 *bacon_video_widget_get_languages (BaconVideoWidget *bvw)
 {
 	GList *list = NULL;
-	int i;
+	int i, num_channels;
 	char lang[XINE_LANG_MAX];
 
-	for(i = 0; i < 32; i++)
+	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), NULL);
+	g_return_val_if_fail (bvw->priv->stream != NULL, NULL);
+
+	if (bvw->priv->mrl == NULL)
+		return NULL;
+
+	num_channels = xine_get_stream_info
+		(bvw->priv->stream, XINE_STREAM_INFO_MAX_AUDIO_CHANNEL);
+
+	if (num_channels < 2)
+		return NULL;
+
+	for(i = 0; i < num_channels; i++)
 	{
 		memset (&lang, 0, sizeof (lang));
 
@@ -3314,6 +3362,11 @@ GList
 		{
 			list = g_list_prepend (list,
 					(gpointer) g_strdup (lang));
+		} else {
+			/* An unnamed language, for example 'Language 2' */
+			list = g_list_prepend (list,
+					(gpointer) g_strdup_printf
+					(_("Language %d"), i + 1));
 		}
 	}
 
@@ -3347,6 +3400,12 @@ GList
 	int i;
 	char lang[XINE_LANG_MAX];
 
+	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), NULL);
+	g_return_val_if_fail (bvw->priv->stream != NULL, NULL);
+
+	if (bvw->priv->mrl == NULL)
+		return NULL;
+
 	for(i = 0; i < 32; i++)
 	{
 		memset (&lang, 0, sizeof (lang));
@@ -3364,12 +3423,18 @@ GList
 int
 bacon_video_widget_get_subtitle (BaconVideoWidget *bvw)
 {
+	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), -2);
+	g_return_val_if_fail (bvw->priv->stream != NULL, -2);
+
 	return xine_get_param (bvw->priv->stream, XINE_PARAM_SPU_CHANNEL);
 }
 
 void
 bacon_video_widget_set_subtitle (BaconVideoWidget *bvw, int subtitle)
 {
+	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+	g_return_if_fail (bvw->priv->stream != NULL);
+
 	xine_set_param (bvw->priv->stream, XINE_PARAM_SPU_CHANNEL, subtitle);
 }
 
