@@ -15,6 +15,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
+ * The Totem project hereby grant permission for non-gpl compatible GStreamer
+ * plugins to be used and distributed together with GStreamer and Totem. This
+ * permission are above and beyond the permissions granted by the GPL license
+ * Totem is covered by.
+ *
+ * Monday 7th February 2005: Christian Schaller: Add excemption clause.
+ * See license_change file for details.
+ *
  */
 
 #ifndef __TOTEM_PRIVATE_H__
@@ -22,6 +30,7 @@
 
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
+#include <libgnomevfs/gnome-vfs.h>
 
 #include "totem-remote.h"
 #include "scrsaver.h"
@@ -47,6 +56,7 @@ typedef enum {
 struct Totem {
 	/* Control window */
 	GladeXML *xml;
+	GladeXML *xml_popup;
 	GtkWidget *win;
 	BaconVideoWidget *bvw;
 	GtkWidget *prefs;
@@ -104,6 +114,9 @@ struct Totem {
 	EggRecentModel *recent_model;
 	EggRecentViewGtk *recent_view;
 
+	/* Monitor to automatically unmount */
+	GnomeVFSVolumeMonitor *monitor;
+
 	/* session */
 	const char *argv0;
 	gint64 seek_to;
