@@ -27,6 +27,7 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "totem-uri.h"
+#include "totem-private.h"
 
 gboolean
 totem_playing_dvd (const char *uri)
@@ -92,3 +93,74 @@ totem_create_full_path (const char *path)
 	return retval;
 }
 
+static void
+totem_action_on_unmount (GnomeVFSVolumeMonitor *vfsvolumemonitor,
+		GnomeVFSVolume *volume, Totem *totem)
+{
+	totem_playlist_clear_with_gnome_vfs_volume (totem->playlist, volume);
+}
+
+void
+totem_setup_file_monitoring (Totem *totem)
+{
+	totem->monitor = gnome_vfs_get_volume_monitor ();
+
+	g_signal_connect (G_OBJECT (totem->monitor),
+			"volume_pre_unmount",
+			G_CALLBACK (totem_action_on_unmount),
+			totem);
+}
+
+/* List from xine-lib's demux_sputext.c */
+static const char *subtitle_ext[] = {
+	"asc",
+	"txt",
+	"sub",
+	"srt",
+	"smi",
+	"ssa"
+};
+
+char *
+totem_uri_get_subtitle_uri (const char *uri)
+{
+	char *suffix, *subtitle, *full;
+	guint len, i;
+
+	if (g_str_has_prefix (uri, "file://") == FALSE) {
+		return NULL;
+	}
+
+	if (strstr (uri, "#subtitle:") != NULL) {
+		return NULL;
+	}
+
+	len = strlen (uri);
+	if (uri[len-4] != '.') {
+		return NULL;
+	}
+
+	full = NULL;
+	subtitle = g_strdup (uri);
+	suffix = subtitle + len - 4;
+	for (i = 0; i < G_N_ELEMENTS(subtitle_ext) ; i++) {
+		char *fname;
+
+		memcpy (suffix + 1, subtitle_ext[i], 3);
+		fname = g_filename_from_uri (subtitle, NULL, NULL);
+
+		if (fname == NULL)
+			continue;
+
+		if (g_file_test (fname, G_FILE_TEST_IS_REGULAR) == FALSE) {
+			g_free (fname);
+			continue;
+		}
+
+		g_free (fname);
+		return subtitle;
+	}
+
+	return NULL;
+}
+
