@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* 
  * Copyright (C) 2001-2004 Bastien Nocera <hadess@hadess.net>
  *
@@ -15,6 +16,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
+ * The Totem project hereby grant permission for non-gpl compatible GStreamer
+ * plugins to be used and distributed together with GStreamer and Totem. This
+ * permission are above and beyond the permissions granted by the GPL license
+ * Totem is covered by.
+ *
+ * Monday 7th February 2005: Christian Schaller: Add excemption clause.
+ * See license_change file for details.
+ *
  */
 
 #include "config.h"
@@ -28,8 +37,6 @@
 #include <stdlib.h>
 #endif /* !HAVE_GTK_ONLY */
 
-#include <libgnomevfs/gnome-vfs.h>
-#include <gconf/gconf-client.h>
 #include <string.h>
 
 /* X11 headers */
@@ -41,8 +48,6 @@
 
 #include "bacon-video-widget.h"
 #include "bacon-video-widget-properties.h"
-#include "rb-ellipsizing-label.h"
-#include "bacon-cd-selection.h"
 #include "totem-statusbar.h"
 #include "totem-time-label.h"
 #include "totem-session.h"
@@ -50,6 +55,7 @@
 #include "totem-sublang.h"
 #include "totem-options.h"
 #include "totem-uri.h"
+#include "totem-interface.h"
 #include "video-utils.h"
 
 #include "egg-recent-view.h"
@@ -79,6 +85,8 @@
 #define ZOOM_IN_OFFSET 1
 #define ZOOM_OUT_OFFSET -1
 
+#define BVW_VBOX_BORDER_WIDTH 1
+
 static const GtkTargetEntry target_table[] = {
 	{ "text/uri-list", 0, 0 },
 	{ "_NETSCAPE_URL", 0, 1 },
@@ -101,6 +109,7 @@ static void playlist_changed_cb (GtkWidget *playlist, Totem *totem);
 static void show_controls (Totem *totem, gboolean was_fullscreen);
 static gboolean totem_is_fullscreen (Totem *totem);
 static void play_pause_set_label (Totem *totem, TotemStates state);
+static gboolean on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event, Totem *totem);
 
 static void
 long_action (void)
@@ -110,62 +119,18 @@ long_action (void)
 }
 
 void
-totem_action_error (char *title, char *reason, Totem *totem)
+totem_action_error (const char *title, const char *reason, Totem *totem)
 {
-	GtkWidget *parent, *error_dialog;
-	char *title_esc, *reason_esc;
-
-	if (reason == NULL)
-		g_warning ("totem_action_error called with reason == NULL");
-
-	if (totem == NULL)
-		parent = NULL;
-	else
-		parent = totem->win;
-
-	title_esc = g_markup_escape_text (title, -1);
-	reason_esc = g_markup_escape_text (reason, -1);
-
-	error_dialog =
-		gtk_message_dialog_new (GTK_WINDOW (parent),
-				GTK_DIALOG_MODAL,
-				GTK_MESSAGE_ERROR,
-				GTK_BUTTONS_OK,
-				"<b>%s</b>\n%s", title_esc, reason_esc);
-	g_free (title_esc);
-	g_free (reason_esc);
-
-	gtk_container_set_border_width (GTK_CONTAINER (error_dialog), 5);
-	gtk_dialog_set_default_response (GTK_DIALOG (error_dialog),
-			GTK_RESPONSE_OK);
-	gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (error_dialog)->label), TRUE);
-	g_signal_connect (G_OBJECT (error_dialog), "destroy", G_CALLBACK
-			(gtk_widget_destroy), error_dialog);
-	g_signal_connect (G_OBJECT (error_dialog), "response", G_CALLBACK
-			(gtk_widget_destroy), error_dialog);
-	gtk_window_set_modal (GTK_WINDOW (error_dialog), TRUE);
-
-	gtk_widget_show (error_dialog);
+	totem_interface_error (title, reason,
+			GTK_WINDOW (totem->win));
 }
 
 static void
-totem_action_error_and_exit (char *title, char *reason, Totem *totem)
+totem_action_error_and_exit (const char *title,
+		const char *reason, Totem *totem)
 {
-	GtkWidget *error_dialog;
-
-	error_dialog =
-		gtk_message_dialog_new (NULL,
-				GTK_DIALOG_MODAL,
-				GTK_MESSAGE_ERROR,
-				GTK_BUTTONS_OK,
-				"<b>%s</b>\n%s", title, reason);
-	gtk_container_set_border_width (GTK_CONTAINER (error_dialog), 5);
-	gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (error_dialog)->label), TRUE);
-	gtk_dialog_set_default_response (GTK_DIALOG (error_dialog),
-			GTK_RESPONSE_OK);
-	gtk_window_set_modal (GTK_WINDOW (error_dialog), TRUE);
-	gtk_dialog_run (GTK_DIALOG (error_dialog));
-
+	totem_interface_error_blocking (title, reason,
+			GTK_WINDOW (totem->win));
 	totem_action_exit (totem);
 }
 
@@ -224,6 +189,8 @@ totem_action_exit (Totem *totem)
 	if (totem->win)
 		gtk_widget_destroy (GTK_WIDGET (totem->win));
 
+	gnome_vfs_shutdown ();
+
 	exit (0);
 }
 
@@ -232,7 +199,7 @@ totem_action_menu_popup (Totem *totem, guint button)
 {
 	GtkWidget *menu;
 
-	menu = glade_xml_get_widget (totem->xml,
+	menu = glade_xml_get_widget (totem->xml_popup,
 			"totem_right_click_menu");
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
 			button, gtk_get_current_event_time ());
@@ -272,19 +239,19 @@ play_pause_set_label (Totem *totem, TotemStates state)
 	case STATE_PLAYING:
 		totem_statusbar_set_text (TOTEM_STATUSBAR (totem->statusbar),
 				_("Playing"));
-		id = "stock-media-pause";
+		id = GTK_STOCK_MEDIA_PAUSE;
 		break;
 	case STATE_PAUSED:
 		totem_statusbar_set_text (TOTEM_STATUSBAR (totem->statusbar),
 				_("Paused"));
-		id = "stock-media-play";
+		id = GTK_STOCK_MEDIA_PLAY;
 		break;
 	case STATE_STOPPED:
 		totem_statusbar_set_text (TOTEM_STATUSBAR (totem->statusbar),
 				_("Stopped"));
 		totem_statusbar_set_time_and_length
 			(TOTEM_STATUSBAR (totem->statusbar), 0, 0);
-		id = "stock-media-play";
+		id = GTK_STOCK_MEDIA_PLAY;
 		break;
 	default:
 		return;
@@ -292,11 +259,9 @@ play_pause_set_label (Totem *totem, TotemStates state)
 
 	image = glade_xml_get_widget (totem->xml,
 			"tmw_play_pause_button_image");
-	gtk_image_set_from_pixbuf (GTK_IMAGE (image),
-			totem_get_named_icon_for_id (id));
+	gtk_image_set_from_stock (GTK_IMAGE (image), id, GTK_ICON_SIZE_BUTTON);
 	image = glade_xml_get_widget (totem->xml, "tcw_pp_button_image");
-	gtk_image_set_from_pixbuf (GTK_IMAGE (image),
-			totem_get_named_icon_for_id (id));
+	gtk_image_set_from_stock (GTK_IMAGE (image), id, GTK_ICON_SIZE_BUTTON);
 
 	totem->state = state;
 }
@@ -336,6 +301,9 @@ totem_action_play (Totem *totem)
 
 	retval = bacon_video_widget_play (totem->bvw,  &err);
 	play_pause_set_label (totem, retval ? STATE_PLAYING : STATE_STOPPED);
+	if (totem_is_fullscreen (totem) != FALSE)
+		totem_scrsaver_disable (totem->scr);
+
 	if (retval == FALSE)
 	{
 		char *msg, *disp;
@@ -430,6 +398,7 @@ void
 totem_action_stop (Totem *totem)
 {
 	bacon_video_widget_stop (totem->bvw);
+	totem_scrsaver_enable (totem->scr);
 }
 
 void
@@ -456,9 +425,12 @@ totem_action_play_pause (Totem *totem)
 	{
 		bacon_video_widget_play (totem->bvw, NULL);
 		play_pause_set_label (totem, STATE_PLAYING);
+		if (totem_is_fullscreen (totem) != FALSE)
+			totem_scrsaver_disable (totem->scr);
 	} else {
 		bacon_video_widget_pause (totem->bvw);
 		play_pause_set_label (totem, STATE_PAUSED);
+		totem_scrsaver_enable (totem->scr);
 	}
 }
 
@@ -473,6 +445,7 @@ totem_action_fullscreen_toggle (Totem *totem)
 		bacon_video_widget_set_fullscreen (totem->bvw, FALSE);
 		gtk_window_unfullscreen (GTK_WINDOW(totem->win));
 		bacon_video_widget_set_show_cursor (totem->bvw, TRUE);
+
 		totem_scrsaver_enable (totem->scr);
 
 		totem->controls_visibility = TOTEM_CONTROLS_VISIBLE;
@@ -495,7 +468,8 @@ totem_action_fullscreen_toggle (Totem *totem)
 		bacon_video_widget_set_fullscreen (totem->bvw, TRUE);
 		bacon_video_widget_set_show_cursor (totem->bvw, FALSE);
 		gtk_window_fullscreen (GTK_WINDOW(totem->win));
-		totem_scrsaver_disable (totem->scr);
+		if (bacon_video_widget_is_playing (totem->bvw) != FALSE)
+			totem_scrsaver_disable (totem->scr);
 
 		totem->controls_visibility = TOTEM_CONTROLS_FULLSCREEN;
 		show_controls (totem, FALSE);
@@ -572,12 +546,10 @@ update_mrl_label (Totem *totem, const char *name)
 			("<span size=\"medium\"><b>%s</b></span>", escaped);
 		g_free (escaped);
 
-		widget = glade_xml_get_widget (totem->xml, "tcw_label_custom");
-		rb_ellipsizing_label_set_markup (RB_ELLIPSIZING_LABEL (widget),
-				text);
+		widget = glade_xml_get_widget (totem->xml, "tcw_title_label");
+		gtk_label_set_markup (GTK_LABEL (widget), text);
 		widget = glade_xml_get_widget (totem->xml, "tmw_title_label");
-		rb_ellipsizing_label_set_markup (RB_ELLIPSIZING_LABEL (widget),
-				text);
+		gtk_label_set_markup (GTK_LABEL (widget), text);
 
 		g_free (text);
 
@@ -597,12 +569,10 @@ update_mrl_label (Totem *totem, const char *name)
 		text = g_strdup_printf
 			("<span size=\"medium\"><b>%s</b></span>",
 			 _("No file"));
-		widget = glade_xml_get_widget (totem->xml, "tcw_label_custom");
-		rb_ellipsizing_label_set_markup (RB_ELLIPSIZING_LABEL (widget),
-				text);
+		widget = glade_xml_get_widget (totem->xml, "tcw_title_label");
+		gtk_label_set_markup (GTK_LABEL (widget), text);
 		widget = glade_xml_get_widget (totem->xml, "tmw_title_label");
-		rb_ellipsizing_label_set_markup (RB_ELLIPSIZING_LABEL (widget),
-				text);
+		gtk_label_set_markup (GTK_LABEL (widget), text);
 
 		g_free (text);
 
@@ -641,7 +611,7 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		widget = glade_xml_get_widget (totem->xml, "tmw_play_menu_item");
 		gtk_widget_set_sensitive (widget, FALSE);
 
-		widget = glade_xml_get_widget (totem->xml, "trcm_play");
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_play");
 		gtk_widget_set_sensitive (widget, FALSE);
 
 		/* Seek bar and seek buttons */
@@ -654,9 +624,9 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		gtk_widget_set_sensitive (widget, FALSE);
 		widget = glade_xml_get_widget (totem->xml, "tmw_volume_down_menu_item");
 		gtk_widget_set_sensitive (widget, FALSE);
-		widget = glade_xml_get_widget (totem->xml, "trcm_volume_up");
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_volume_up");
 		gtk_widget_set_sensitive (widget, FALSE);
-		widget = glade_xml_get_widget (totem->xml, "trcm_volume_down");
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_volume_down");
 		gtk_widget_set_sensitive (widget, FALSE);
 
 		/* Control popup */
@@ -667,9 +637,9 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		gtk_widget_set_sensitive (widget, FALSE);
 		widget = glade_xml_get_widget (totem->xml, "tcw_next_button"); 
 		gtk_widget_set_sensitive (widget, FALSE);
-		widget = glade_xml_get_widget (totem->xml, "trcm_next_chapter");
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_next_chapter");
 		gtk_widget_set_sensitive (widget, FALSE);
-		widget = glade_xml_get_widget (totem->xml, "trcm_previous_chapter");
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_previous_chapter");
 		gtk_widget_set_sensitive (widget, FALSE);
 		widget = glade_xml_get_widget (totem->xml, "tcw_volume_hbox");
 		gtk_widget_set_sensitive (widget, FALSE);
@@ -686,6 +656,7 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		update_mrl_label (totem, NULL);
 	} else {
 		gboolean caps;
+		char *subtitle_uri;
 		GError *err = NULL;
 
 		if (bacon_video_widget_get_logo_mode (totem->bvw) != FALSE)
@@ -694,14 +665,16 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 			bacon_video_widget_set_logo_mode (totem->bvw, FALSE);
 		}
 
-		retval = bacon_video_widget_open (totem->bvw, mrl, &err);
+		subtitle_uri = totem_uri_get_subtitle_uri (mrl);
+		retval = bacon_video_widget_open_with_subtitle (totem->bvw,
+				mrl, subtitle_uri, &err);
 		totem->mrl = g_strdup (mrl);
 
 		/* Play/Pause */
 		gtk_widget_set_sensitive (totem->pp_button, TRUE);
 		widget = glade_xml_get_widget (totem->xml, "tmw_play_menu_item");
 		gtk_widget_set_sensitive (widget, TRUE);
-		widget = glade_xml_get_widget (totem->xml, "trcm_play");
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_play");
 		gtk_widget_set_sensitive (widget, TRUE);
 		gtk_widget_set_sensitive (totem->fs_pp_button, TRUE);
 
@@ -719,6 +692,10 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		gtk_widget_set_sensitive (widget, caps);
 		widget = glade_xml_get_widget (totem->xml, "tcw_volume_hbox");
 		gtk_widget_set_sensitive (widget, caps);
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_volume_up");                gtk_widget_set_sensitive (widget, caps);
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_volume_down");
+		gtk_widget_set_sensitive (widget, caps);
+
 
 		/* Take a screenshot */
 		widget = glade_xml_get_widget (totem->xml,
@@ -1128,7 +1105,7 @@ on_recent_file_activate (EggRecentViewGtk *view, EggRecentItem *item,
 
 /* This is only called when we are playing a DVD */
 static void
-on_title_change_event (GtkWidget *win, const char *string, Totem *totem)
+on_title_change_event (BaconVideoWidget *bvw, const char *string, Totem *totem)
 {
 	update_mrl_label (totem, string);
 	totem_playlist_set_title (TOTEM_PLAYLIST (totem->playlist), string);
@@ -1202,6 +1179,9 @@ update_seekable (Totem *totem, gboolean seekable)
 {
 	GtkWidget *widget;
 
+	if (totem->seekable == seekable)
+		return;
+
 	/* Check if the stream is seekable */
 	gtk_widget_set_sensitive (totem->seek, seekable);
 	gtk_widget_set_sensitive (totem->fs_seek, seekable);
@@ -1218,16 +1198,15 @@ update_seekable (Totem *totem, gboolean seekable)
 	widget = glade_xml_get_widget (totem->xml,
 			"tmw_skip_backwards_menu_item");
 	gtk_widget_set_sensitive (widget, seekable);
-	widget = glade_xml_get_widget (totem->xml, "trcm_skip_forward");
+	widget = glade_xml_get_widget (totem->xml_popup, "trcm_skip_forward");
 	gtk_widget_set_sensitive (widget, seekable);
-	widget = glade_xml_get_widget (totem->xml, "trcm_skip_backwards");
+	widget = glade_xml_get_widget (totem->xml_popup,
+			"trcm_skip_backwards");
 	gtk_widget_set_sensitive (widget, seekable);
 	widget = glade_xml_get_widget (totem->xml, "tmw_skip_to_menu_item");
 	gtk_widget_set_sensitive (widget, seekable);
 	if (totem->skipto)
 		totem_skipto_set_seekable (totem->skipto, seekable);
-
-	totem->seekable = seekable;
 }
 
 static void
@@ -1576,31 +1555,19 @@ static void
 on_open_location1_activate (GtkButton *button, Totem *totem)
 {
 	GladeXML *glade;
-	char *filename, *mrl;
-	GtkWidget *dialog, *entry;
+	char *mrl;
+	GtkWidget *dialog, *entry, *gentry;
 	int response;
 	const char *filenames[2];
 
-	filename = g_build_filename (DATADIR,
-			"totem", "uri.glade", NULL);
-	if (g_file_test (filename, G_FILE_TEST_EXISTS) == FALSE)
-	{
-		g_free (filename);
-		totem_action_error (_("Couldn't load the 'Open Location...' interface."), _("Make sure that Totem is properly installed."), totem);
-		return;
-	}
-
-	glade = glade_xml_new (filename, NULL, NULL);
+	glade = totem_interface_load ("uri.glade", _("Open Location..."),
+			FALSE, GTK_WINDOW (totem->win));
 	if (glade == NULL)
-	{
-		g_free (filename);
-		totem_action_error (_("Couldn't load the 'Open Location...' interface."), _("Make sure that Totem is properly installed."), totem);
 		return;
-	}
 
-	g_free (filename);
 	dialog = glade_xml_get_widget (glade, "open_uri_dialog");
 	entry = glade_xml_get_widget (glade, "uri");
+	gentry = glade_xml_get_widget (glade, "uri_list");
 
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 
@@ -1618,6 +1585,9 @@ on_open_location1_activate (GtkButton *button, Totem *totem)
 			mrl = totem_playlist_get_current_mrl (totem->playlist);
 			totem_action_set_mrl_and_play (totem, mrl);
 			g_free (mrl);
+
+			gnome_entry_append_history (GNOME_ENTRY (gentry),
+					TRUE, uri);
 		}
 	}
 
@@ -1791,7 +1761,7 @@ show_controls (Totem *totem, gboolean was_fullscreen)
 	menubar = glade_xml_get_widget (totem->xml, "tmw_menubar");
 	controlbar = glade_xml_get_widget (totem->xml, "tmw_controls_vbox");
 	statusbar = glade_xml_get_widget (totem->xml, "tmw_statusbar");
-	item = glade_xml_get_widget (totem->xml, "trcm_show_controls");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_show_controls");
 	bvw_vbox = glade_xml_get_widget (totem->xml, "tmw_bvw_vbox");
 	widget = GTK_WIDGET (totem->bvw);
 
@@ -1807,7 +1777,8 @@ show_controls (Totem *totem, gboolean was_fullscreen)
 		gtk_widget_show (controlbar);
 		gtk_widget_show (statusbar);
 		gtk_widget_hide (item);
-		gtk_container_set_border_width (GTK_CONTAINER (bvw_vbox), 1);
+		gtk_container_set_border_width (GTK_CONTAINER (bvw_vbox),
+				BVW_VBOX_BORDER_WIDTH);
 
 		if (was_fullscreen == FALSE)
 		{
@@ -1889,6 +1860,7 @@ on_help_activate (GtkButton *button, Totem *totem)
 		g_error_free (err);
 	}
 }
+#endif /* !HAVE_GTK_ONLY */
 
 static void
 on_about1_activate (GtkButton *button, Totem *totem)
@@ -1901,9 +1873,14 @@ on_about1_activate (GtkButton *button, Totem *totem)
 		"Julien Moutte <julien@moutte.net> (GStreamer backend)",
 		NULL
 	};
-	const char *documenters[] = { NULL };
-	const char *translator_credits = _("translator_credits");
+	const char *artists[] = { "Jakub Steiner <jimmac@ximian.com>", NULL };
+	const char *documenters[] =
+	{
+		"Chee Bin Hoh <cbhoh@gnome.org>",
+		NULL
+	};
 	char *backend_version, *description;
+	char *filename;
 
 	if (totem->about != NULL)
 	{
@@ -1911,32 +1888,26 @@ on_about1_activate (GtkButton *button, Totem *totem)
 		return;
 	}
 
-	{
-		char *filename = NULL;
-
-		filename = gnome_program_locate_file (NULL,
-				GNOME_FILE_DOMAIN_APP_DATADIR,
-				"totem/media-player-48.png",
-				TRUE, NULL);
-
-		if (filename != NULL)
-		{
-			pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
-			g_free (filename);
-		}
-	}
+	filename = g_build_filename (DATADIR,
+			"totem", "media-player-48.png", NULL);
+	pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
+	g_free (filename);
 
 	backend_version = bacon_video_widget_get_backend_name (totem->bvw);
 	description = g_strdup_printf (_("Movie Player using %s"),
 				backend_version);
 
-	totem->about = gnome_about_new(_("Totem"), VERSION,
-			"Copyright \xc2\xa9 2002-2004 Bastien Nocera",
-			(const char *)description,
-			(const char **)authors,
-			(const char **)documenters,
-			strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-			pixbuf);
+	totem->about = g_object_new (GTK_TYPE_ABOUT_DIALOG,
+			"name", _("Totem"),
+			"version", VERSION,
+			"copyright", _("Copyright \xc2\xa9 2002-2005 Bastien Nocera"),
+			"comments", description,
+			"authors", authors,
+			"documenters", documenters,
+			"artists", artists, 
+			"translator-credits", _("translator-credits"),
+			"logo", pixbuf,
+			NULL);
 
 	g_free (backend_version);
 	g_free (description);
@@ -1948,10 +1919,11 @@ on_about1_activate (GtkButton *button, Totem *totem)
 			(gpointer *)&totem->about);
 	gtk_window_set_transient_for (GTK_WINDOW (totem->about),
 			GTK_WINDOW (totem->win));
+	g_signal_connect (G_OBJECT (totem->about), "response",
+			G_CALLBACK (gtk_widget_destroy), NULL);
 
 	gtk_widget_show(totem->about);
 }
-#endif /* !HAVE_GTK_ONLY */
 
 static void
 on_take_screenshot1_activate (GtkButton *button, Totem *totem)
@@ -2116,8 +2088,8 @@ on_skip_to1_activate (GtkButton *button, Totem *totem)
 	totem->skipto = TOTEM_SKIPTO (totem_skipto_new (filename));
 	g_free (filename);
 
-	g_signal_connect (G_OBJECT (totem->skipto),
-			"delete-event", G_CALLBACK (gtk_widget_destroy), NULL);
+	g_signal_connect (G_OBJECT (totem->skipto), "delete-event",
+			G_CALLBACK (gtk_widget_destroy), NULL);
 	g_signal_connect (G_OBJECT (totem->skipto), "response",
 			G_CALLBACK (commit_hide_skip_to), totem);
 	g_object_add_weak_pointer (G_OBJECT (totem->skipto),
@@ -2166,6 +2138,8 @@ on_volume_max_button (GtkButton *button, Totem *totem)
 static void
 totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 {
+	gboolean handled = TRUE;
+
 	switch (cmd) {
 	case TOTEM_REMOTE_COMMAND_PLAY:
 		totem_action_play (totem);
@@ -2238,8 +2212,12 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 		}
 		break;
 	default:
+		handled = FALSE;
 		break;
 	}
+
+	if (handled != FALSE)
+		on_video_motion_notify_event (NULL, NULL, totem);
 }
 
 #ifdef HAVE_REMOTE
@@ -2374,7 +2352,7 @@ move_popups (Totem *totem)
 	if (gtk_widget_get_direction (totem->exit_popup) == GTK_TEXT_DIR_RTL)
 	{
 		gtk_window_move (GTK_WINDOW (totem->exit_popup),
-				totem->fullscreen_rect.width - exit_width,
+				totem->fullscreen_rect.x,
 				totem->fullscreen_rect.y);
 		gtk_window_move (GTK_WINDOW (totem->control_popup),
 				totem->fullscreen_rect.width - control_width,
@@ -2382,7 +2360,7 @@ move_popups (Totem *totem)
 				- control_height);
 	} else {
 		gtk_window_move (GTK_WINDOW (totem->exit_popup),
-				totem->fullscreen_rect.x,
+				totem->fullscreen_rect.width - exit_width,
 				totem->fullscreen_rect.y);
 		gtk_window_move (GTK_WINDOW (totem->control_popup),
 				totem->fullscreen_rect.x,
@@ -2409,7 +2387,7 @@ popup_timeout_remove (Totem *totem)
 {
 	if (totem->popup_timeout != 0)
 	{
-		gtk_timeout_remove (totem->popup_timeout);
+		g_source_remove (totem->popup_timeout);
 		totem->popup_timeout = 0;
 	}
 }
@@ -2438,8 +2416,8 @@ on_mouse_click_fullscreen (GtkWidget *widget, Totem *totem)
 {
 	popup_timeout_remove (totem);
 
-	totem->popup_timeout = gtk_timeout_add (2000,
-		(GtkFunction) popup_hide, totem);
+	totem->popup_timeout = g_timeout_add (2000,
+		(GSourceFunc) popup_hide, totem);
 }
 
 static gboolean
@@ -2470,8 +2448,8 @@ on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event,
 	gtk_widget_show_all (totem->control_popup);
 	bacon_video_widget_set_show_cursor (totem->bvw, TRUE);
 
-	totem->popup_timeout = gtk_timeout_add (5000,
-			(GtkFunction) popup_hide, totem);
+	totem->popup_timeout = g_timeout_add (5000,
+			(GSourceFunc) popup_hide, totem);
 	totem->popup_in_progress = FALSE;
 
 	return FALSE;
@@ -2508,6 +2486,7 @@ on_eos_event (GtkWidget *widget, Totem *totem)
 		mrl = totem_playlist_get_current_mrl (totem->playlist);
 		totem_action_stop (totem);
 		totem_action_set_mrl_with_warning (totem, mrl, FALSE);
+		bacon_video_widget_pause (totem->bvw);
 		g_free (mrl);
 	} else {
 		totem_action_next (totem);
@@ -2819,7 +2798,7 @@ update_buttons (Totem *totem)
 	item = glade_xml_get_widget (totem->xml,
 			"tmw_previous_chapter_menu_item");
 	gtk_widget_set_sensitive (item, has_item);
-	item = glade_xml_get_widget (totem->xml, "trcm_previous_chapter");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_previous_chapter");
 	gtk_widget_set_sensitive (item, has_item);
 
 	/* Next */
@@ -2837,7 +2816,7 @@ update_buttons (Totem *totem)
 	gtk_widget_set_sensitive (item, has_item);
 	item = glade_xml_get_widget (totem->xml, "tmw_next_chapter_menu_item");
 	gtk_widget_set_sensitive (item, has_item);
-	item = glade_xml_get_widget (totem->xml, "trcm_next_chapter");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_next_chapter");
 	gtk_widget_set_sensitive (item, has_item);
 }
 
@@ -2854,7 +2833,8 @@ totem_setup_window (Totem *totem)
 		return;
 
 	item = glade_xml_get_widget (totem->xml, "tmw_bvw_vbox");
-	gtk_widget_set_size_request (item, w, h);
+	gtk_widget_set_size_request (item,
+			w + BVW_VBOX_BORDER_WIDTH, h + BVW_VBOX_BORDER_WIDTH);
 }
 
 static void
@@ -2954,15 +2934,17 @@ totem_callback_connect (Totem *totem)
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_quit1_activate), totem);
 
+	/* Help */
+	item = glade_xml_get_widget (totem->xml, "tmw_about_menu_item");
+	g_signal_connect (G_OBJECT (item), "activate",
+			G_CALLBACK (on_about1_activate), totem);
+
 #ifndef HAVE_GTK_ONLY
 	item = glade_xml_get_widget (totem->xml, "tmw_contents_menu_item");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_help_activate), totem);
-	item = glade_xml_get_widget (totem->xml, "tmw_about_menu_item");
-	g_signal_connect (G_OBJECT (item), "activate",
-			G_CALLBACK (on_about1_activate), totem);
 #else
-	item = glade_xml_get_widget (totem->xml, "tmw_menu_item_help");
+	item = glade_xml_get_widget (totem->xml, "tmw_contents_menu_item");
 	gtk_widget_hide (item);
 #endif /* !HAVE_GTK_ONLY */
 
@@ -2992,28 +2974,28 @@ totem_callback_connect (Totem *totem)
 			G_CALLBACK (on_show_controls1_activate), totem);
 
 	/* Popup menu */
-	item = glade_xml_get_widget (totem->xml, "trcm_play");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_play");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_play1_activate), totem);
-	item = glade_xml_get_widget (totem->xml, "trcm_next_chapter");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_next_chapter");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_next_button_clicked), totem);
-	item = glade_xml_get_widget (totem->xml, "trcm_previous_chapter");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_previous_chapter");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_previous_button_clicked), totem);
-	item = glade_xml_get_widget (totem->xml, "trcm_skip_forward");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_skip_forward");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_skip_forward1_activate), totem);
-	item = glade_xml_get_widget (totem->xml, "trcm_skip_backwards");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_skip_backwards");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_skip_backwards1_activate), totem);
-	item = glade_xml_get_widget (totem->xml, "trcm_volume_up");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_volume_up");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_volume_up1_activate), totem);
-	item = glade_xml_get_widget (totem->xml, "trcm_volume_down");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_volume_down");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_volume_down1_activate), totem);
-	item = glade_xml_get_widget (totem->xml, "trcm_show_controls");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_show_controls");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_show_controls2_activate), totem);
 
@@ -3213,7 +3195,7 @@ totem_callback_connect (Totem *totem)
 	totem_set_default_icons (totem);
 
 	/* Update the UI */
-	gtk_timeout_add (600, (GtkFunction) gui_update_cb, totem);
+	g_timeout_add (600, (GSourceFunc) gui_update_cb, totem);
 }
 
 static void
@@ -3245,11 +3227,6 @@ video_widget_create (Totem *totem)
 	totem_preferences_tvout_setup (totem);
 	totem_preferences_visuals_setup (totem);
 
-	bacon_video_widget_set_volume (totem->bvw,
-			gconf_client_get_int (totem->gc,
-				GCONF_PREFIX"/volume", NULL));
-	update_volume_sliders (totem);
-
 	/* Let's set a name. Will make debugging easier */
 	gtk_widget_set_name (GTK_WIDGET(totem->bvw), "bvw");
 
@@ -3322,28 +3299,11 @@ video_widget_create (Totem *totem)
 	gtk_widget_show (GTK_WIDGET (totem->bvw));
 
 	gtk_widget_set_size_request (container, -1, -1);
-}
 
-GtkWidget *
-label_create (void)
-{
-	GtkWidget *label;
-	char *text;
-
-	label = rb_ellipsizing_label_new ("");
-	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
-	gtk_widget_show (label);
-
-	/* Set default */
-	text = g_strdup_printf
-		("<span size=\"medium\"><b>%s</b></span>",
-		 _("No file"));
-	rb_ellipsizing_label_set_markup (RB_ELLIPSIZING_LABEL (label), text);
-	g_free (text);
-
-	return label;
+	bacon_video_widget_set_volume (totem->bvw,
+			gconf_client_get_int (totem->gc,
+				GCONF_PREFIX"/volume", NULL));
+	update_volume_sliders (totem);
 }
 
 GtkWidget *
@@ -3401,12 +3361,6 @@ totem_setup_recent (Totem *totem)
 			G_CALLBACK (on_recent_file_activate), totem);
 }
 
-GConfClient *
-totem_get_gconf_client (Totem *totem)
-{
-	return totem->gc;
-}
-
 static void
 totem_message_connection_receive_cb (const char *msg, Totem *totem)
 {
@@ -3436,7 +3390,6 @@ main (int argc, char **argv)
 	Totem *totem;
 	char *filename;
 	GConfClient *gc;
-	GdkPixbuf *pix;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -3451,18 +3404,17 @@ main (int argc, char **argv)
 	}
 
 	g_thread_init (NULL);
-	gdk_threads_init ();
 
+#ifdef HAVE_GTK_ONLY
 	gtk_init (&argc, &argv);
-
-#ifndef HAVE_GTK_ONLY
+#else
 	gnome_program_init ("totem", VERSION,
 			LIBGNOMEUI_MODULE,
 			argc, argv,
 			GNOME_PARAM_APP_DATADIR, DATADIR,
 			GNOME_PARAM_POPT_TABLE, totem_options_get_options (),
 			GNOME_PARAM_NONE);
-#endif /* !HAVE_GTK_ONLY */
+#endif /* HAVE_GTK_ONLY */
 
 	gnome_vfs_init ();
 
@@ -3475,18 +3427,6 @@ main (int argc, char **argv)
 	gnome_authentication_manager_init ();
 #endif /* !HAVE_GTK_ONLY */
 
-	if (g_file_test ("../data/totem.glade", G_FILE_TEST_EXISTS) != FALSE)
-		filename = g_strdup ("../data/totem.glade");
-	else
-		filename = g_build_filename (DATADIR,
-				"totem", "totem.glade", NULL);
-
-	if (g_file_test (filename, G_FILE_TEST_EXISTS) == FALSE)
-	{
-		g_free (filename);
-		totem_action_error_and_exit (_("Couldn't load the main interface (totem.glade)."), _("Make sure that Totem is properly installed."), NULL);
-	}
-
 	totem = g_new0 (Totem, 1);
 
 	/* IPC stuff */
@@ -3507,42 +3447,28 @@ main (int argc, char **argv)
 	totem->gc = gc;
 
 	/* Main window */
-	totem->xml = glade_xml_new (filename, NULL, NULL);
+	totem->xml = totem_interface_load ("totem.glade", _("main window"),
+			TRUE, NULL);
 	if (totem->xml == NULL)
-	{
-		g_free (filename);
-		totem_action_error_and_exit (_("Couldn't load the main interface (totem.glade)."), _("Make sure that Totem is properly installed."), NULL);
-	}
-	g_free (filename);
+		totem_action_exit (NULL);
+	totem->xml_popup = totem_interface_load ("popup.glade",
+			_("video popup menu"),
+			TRUE, NULL);
+	if (totem->xml_popup == NULL)
+		totem_action_exit (NULL);
 
 	totem->win = glade_xml_get_widget (totem->xml, "totem_main_window");
-	filename = g_build_filename (DATADIR,
-			"totem", "media-player-48.png", NULL);
+	filename = totem_interface_get_full_path ("media-player-48.png");
 	gtk_window_set_default_icon_from_file (filename, NULL);
 	g_free (filename);
 
 	totem_named_icons_init (totem, FALSE);
 
 	/* The playlist */
-	filename = g_build_filename (DATADIR,
-			"totem", "playlist-playing.png", NULL);
-	pix = gdk_pixbuf_new_from_file (filename, NULL);
-	g_free (filename);
-
-	filename = g_build_filename (DATADIR,
-			"totem", "playlist.glade", NULL);
-	totem->playlist = TOTEM_PLAYLIST (totem_playlist_new (filename, pix));
-	g_free (filename);
+	totem->playlist = TOTEM_PLAYLIST (totem_playlist_new ());
 
 	if (totem->playlist == NULL)
-	{
-		totem_action_error_and_exit (_("Couldn't load the interface for the playlist."), _("Make sure that Totem is properly installed."), totem);
-	}
-	filename = g_build_filename (DATADIR,
-			"totem", "playlist-24.png", NULL);
-	gtk_window_set_icon_from_file (GTK_WINDOW (totem->playlist),
-			filename, NULL);
-	g_free (filename);
+		totem_action_exit (totem);
 
 	/* The rest of the widgets */
 	totem->state = STATE_STOPPED;
@@ -3580,6 +3506,7 @@ main (int argc, char **argv)
 	totem_setup_recent (totem);
 	totem_callback_connect (totem);
 	totem_setup_window (totem);
+	totem_setup_file_monitoring (totem);
 
 	/* Show ! gtk_main_iteration trickery to show all the widgets
 	 * we have so far */
